@@ -35,6 +35,7 @@
 #include "medusa.h"
 #include "medusa-trace.h"
 #include "medusa-net.h"
+#include "uthash.h"
 #include <pthread.h>
 
 #ifdef HAVE_LIBSSL
@@ -42,13 +43,34 @@
   #include <openssl/err.h>
 #endif
 
-int nUseSSL = 0;
-char ipaddr_str[INET_ADDRSTRLEN];
-
 #ifdef HAVE_LIBSSL
-  SSL *ssl = NULL;
-  SSL_CTX *sslContext = NULL;
-  RSA *rsa = NULL;
+
+/*
+  Original code utilized global variables for SSL socket support.
+  Unfortunately, this could potentially be an issue in cases where
+  modules utilize both SSL and non-SSL within a session (e.g. FTPS). 
+  It's possible one thread could connect to a service in the clear 
+  while another requires SSL. The global variable obviously makes this 
+  impossible.
+
+  Ideally, this data should be unique to thread and tracked there.
+  However, I believe implementing this would require rewriting all
+  send/receive calls. For now, we are utilizing a hash of socket id
+  to SSL information structure for tracking which connections require
+  SSL. 
+*/
+struct SSLSOCKETINFO {
+  int id;                    /* key --> socket id */
+  int nUseSSL;
+  SSL *ssl;
+  SSL_CTX *sslContext;
+
+  UT_hash_handle hh;         /* required for UThash */
+};
+
+struct SSLSOCKETINFO *psSSLSocketInfo = NULL; 
+pthread_mutex_t ptmSSLMutex;
+
 #endif
 
 // Modules can call this function to set up the sConnectParams structure needed for connection functions.
@@ -266,10 +288,13 @@ int medusaConnectInternal(unsigned long nHost, int nPort, int nProtocol, int nTy
   return ret;
 }
 
-
 #ifdef HAVE_LIBSSL
 RSA *sslTempRSACallback(SSL * ssl, int export, int keylength)
 {
+  /* "rsa" was previously global... do we ever need to return a previously
+     generated value? */
+  RSA *rsa = NULL;
+
   if (rsa == NULL)
     rsa = RSA_generate_key(512, RSA_F4, NULL, NULL);
   return rsa;
@@ -278,6 +303,9 @@ RSA *sslTempRSACallback(SSL * ssl, int export, int keylength)
 int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
 {
   int err;
+  struct SSLSOCKETINFO *s;
+  SSL *ssl = NULL;
+  SSL_CTX *sslContext = NULL;
 
   SSL_load_error_strings();
   SSLeay_add_ssl_algorithms();
@@ -329,6 +357,16 @@ int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
 
   writeError(ERR_DEBUG, "SSL negotiated cipher: %s", SSL_get_cipher(ssl));
 
+  s = malloc(sizeof(struct SSLSOCKETINFO));
+  memset(s, 0, sizeof(struct SSLSOCKETINFO));
+  s->id = hSocket;
+  s->nUseSSL = 1;
+  s->ssl = ssl;
+  s->sslContext = sslContext;
+  pthread_mutex_lock(&ptmSSLMutex);
+  HASH_ADD_INT( psSSLSocketInfo, id, s ); 
+  pthread_mutex_unlock(&ptmSSLMutex);
+
   return hSocket;
 }
 #endif
@@ -336,9 +374,30 @@ int medusaConnectSSLInternal(sConnectParams* pParams, int hSocket)
 int medusaReceiveInternal(int socket, char *buf, int length)
 {
 #ifdef HAVE_LIBSSL
-  if (nUseSSL)
+  int err;
+  struct SSLSOCKETINFO *s;
+
+  HASH_FIND_INT( psSSLSocketInfo, &socket, s );
+  if ((s != NULL) && (s->nUseSSL))
   {
-    int nRet = SSL_read(ssl, buf, length);
+    int nRet = SSL_read(s->ssl, buf, length);
+    
+    if (nRet <= 0)
+    {
+      err = SSL_get_error(s->ssl, nRet);
+      switch(err)
+      {
+        case SSL_ERROR_ZERO_RETURN:
+          writeError(ERR_ERROR, "The TLS/SSL connection has been closed.");
+          break;
+        case SSL_ERROR_SSL:
+          writeError(ERR_ERROR, "A failure in the SSL library occurred, usually a protocol error.");
+          break;
+        default:
+          writeError(ERR_ERROR, "Unknown TLS/SSL error occurred (%d - %s)", err, ERR_error_string(err, NULL));
+      }
+    }
+
     return nRet;
   }
   else
@@ -394,7 +453,7 @@ char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsT
     free(szBufReceive);
     return NULL;
   }
-
+  
   /* check for any addition data which may have been sent */
   while (medusaDataReadyTimed(socket, nReceiveDelay2sec, nReceiveDelay2usec) > 0)
   {
@@ -408,15 +467,18 @@ char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsT
       break;
     }
    
-    if (nBufReceive + nBufReceiveTmp > BUFFER_SIZE)
+    if (nBufReceive + nBufReceiveTmp > BUFFER_SIZE) 
+    {
+      writeError(ERR_DEBUG, "Additional data received. Increasing receive buffer %d bytes.", nBufReceiveTmp);
       szBufReceive = realloc(szBufReceive, nBufReceive + nBufReceiveTmp + 1);
+    }
     memcpy(szBufReceive + nBufReceive, szBufReceiveTmp, nBufReceiveTmp);
     nBufReceive += nBufReceiveTmp;
 
     free(szBufReceiveTmp);
   }
 
-  szBufReceive[nBufReceive + nBufReceiveTmp] = 0; /* explicit NULL termination */
+  szBufReceive[nBufReceive] = 0; /* explicit NULL termination */
 
   /* convert NULLS to spaces */
   if (nConvertNullsToSpaces != 0)
@@ -433,9 +495,12 @@ char* medusaReceiveDataInternal(int socket, int* nBufferSize, int nConvertNullsT
 int medusaSendInternal(int socket, char *buf, int size, int options)
 {
 #ifdef HAVE_LIBSSL
-  if (nUseSSL)
+  struct SSLSOCKETINFO *s;
+
+  HASH_FIND_INT( psSSLSocketInfo, &socket, s );
+  if ((s != NULL) && (s->nUseSSL))
   {
-    return SSL_write(ssl, buf, size);
+    return SSL_write(s->ssl, buf, size);
   }
   else
   {
@@ -466,9 +531,14 @@ int medusaConnect(sConnectParams* pParams)
 int medusaConnectSSL(sConnectParams* pParams)
 {
 #ifdef HAVE_LIBSSL
-  nUseSSL = 1;
-  pParams->nUseSSL = 1;
-  return (medusaConnectSSLInternal(pParams, -1));
+  int hSocket;
+
+  hSocket = medusaConnectSSLInternal(pParams, -1);
+
+  if (hSocket > 0)
+    pParams->nUseSSL = 1;
+
+  return hSocket;
 #else
   writeError(ERR_ERROR, "Trying to connect via SSL, but medusa was not compiled with OPENSSL support. Using non-SSL connection.");
   pParams->nUseSSL = 0;
@@ -481,9 +551,17 @@ int medusaConnectSSL(sConnectParams* pParams)
 int medusaConnectSocketSSL(sConnectParams* pParams, int hSocket)
 {
 #ifdef HAVE_LIBSSL
-  nUseSSL = 1;
-  pParams->nUseSSL = 1;
-  return (medusaConnectSSLInternal(pParams, hSocket));
+  if (hSocket > 0)
+  {
+    pParams->nUseSSL = 1;
+    return (medusaConnectSSLInternal(pParams, hSocket));
+  }
+  else
+  {
+    writeError(ERR_ERROR, "Invalid socket handle.");
+    pParams->nUseSSL = 0;
+    return FAILURE;
+  }
 #else
   writeError(ERR_ERROR, "Trying to connect via SSL, but medusa was not compiled with OPENSSL support.");
   pParams->nUseSSL = 0;
@@ -506,13 +584,31 @@ int medusaConnectUDP(sConnectParams* pParams)
   return (medusaConnect(pParams));
 }
 
-int medusaDisconnect(int socket)
+int medusaDisconnect(int hSocket)
 {
-  close(socket);
+#ifdef HAVE_LIBSSL
+  struct SSLSOCKETINFO *s;
 
+  if (hSocket <= 0)
+    return -1;
+
+  /* Remove socket's SSL informational structure (if it exists) */
+  HASH_FIND_INT( psSSLSocketInfo, &hSocket, s );
+  if (s != NULL)
+  {
+    pthread_mutex_lock(&ptmSSLMutex);
+    HASH_DEL( psSSLSocketInfo, s );
+    pthread_mutex_unlock(&ptmSSLMutex);
+  }
+
+  close(hSocket);
   writeError(ERR_DEBUG, "Disconnect successful");
-
   return -1;
+#else
+  close(hSocket);
+  writeError(ERR_DEBUG, "Disconnect successful");
+  return -1;
+#endif
 }
 
 int medusaDataReadyWritingTimed(int socket, time_t sec, time_t usec)
@@ -551,6 +647,15 @@ int medusaDataReady(int socket)
   return (medusaDataReadyTimed(socket, 0, 0));
 }
 
+/* Check socket status. Return 1 if connection is still valid. */
+int medusaCheckSocket(int socket)
+{
+  if (medusaDataReadyTimed(socket, 0, 1) == 0)
+    return 1;
+  else
+    return 0;
+}
+
 int medusaReceive(int socket, char *buf, int length)
 {
   int ret;
