@@ -41,7 +41,7 @@
 #define MODULE_AUTHOR  "fizzgig <fizzgig@foofus.net>"
 #define	MODULE_SUMMARY_USAGE	"Brute force module for HTTP"
 #define MODULE_VERSION		"1.3.0"
-#define MODULE_VERSION_SVN "$Id: http.c 606 2006-10-19 20:22:41Z jmk $"
+#define MODULE_VERSION_SVN "$Id: http.c 812 2007-08-28 21:20:17Z jmk $"
 #define MODULE_SUMMARY_FORMAT	"%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -59,7 +59,7 @@
 #define AUTH_NTLM 3
 
 typedef struct __MODULE_DATA {
-  char *szDomain;
+  //char *szDomain;
   char *szDir;
   char *szHostHeader;
   char *szUserAgent;
@@ -74,8 +74,6 @@ enum HTTP_STATE
   HSTATE_EXITING
 };
 
-int nOptions = 0;
-
 // Forward declarations
 int getAuthType(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPort);
 int tryLogin(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int nPort, char* szLogin, char* szPassword);
@@ -104,7 +102,6 @@ void summaryUsage(char **ppszSummary)
   {
     writeError(ERR_ERROR, "%s reports an error in summaryUsage() : ppszSummary must be NULL when called", MODULE_NAME);
   }
-
 }
 
 /* Display module usage information */
@@ -127,9 +124,9 @@ int go(sLogin* logins, int argc, char *argv[])
   psSessionData = malloc(sizeof(_MODULE_DATA));
   memset(psSessionData, 0, sizeof(_MODULE_DATA));
 
-  psSessionData->szDomain = malloc(strlen("FOO") + 1);
-  memset(psSessionData->szDomain, 0, strlen("FOO") + 1);
-  sprintf(psSessionData->szDomain, "FOO"); 
+  //psSessionData->szDomain = malloc(strlen("FOO") + 1);
+  //memset(psSessionData->szDomain, 0, strlen("FOO") + 1);
+  //sprintf(psSessionData->szDomain, "FOO"); 
  
   if ( !( 0 <= argc <= 2) )
   {
@@ -197,7 +194,6 @@ int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
   int hSocket = -1;
   enum HTTP_STATE nState = HSTATE_NEW;
   char* bufReceive;
-  int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
   int i = 0;
   char *pPass;
   int nPort, nBufLength = 0;
@@ -233,8 +229,6 @@ int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
     switch(nState)
     {
     case HSTATE_NEW:
-      nFirstPass = 0;
-
       // Already have an open socket - close it
       if (hSocket > 0)
         medusaDisconnect(hSocket);
@@ -261,10 +255,10 @@ int initHttp(_MODULE_DATA *_psSessionData, sLogin* login)
 
       if (!_psSessionData->szHostHeader) {
         
-        nBufLength = strlen(login->psServer->pHostIP) + 1 + log(params.nPort) + 1;
+        nBufLength = strlen(login->psServer->psHost->pHost) + 1 + log(params.nPort) + 1;
         _psSessionData->szHostHeader = malloc(nBufLength + 1);
         memset(_psSessionData->szHostHeader, 0, nBufLength + 1);
-        sprintf(_psSessionData->szHostHeader, "%s:%d", login->psServer->pHostIP, params.nPort);
+        sprintf(_psSessionData->szHostHeader, "%s:%d", login->psServer->psHost->pHost, params.nPort);
       }
       
       if (!_psSessionData->szUserAgent) {
@@ -338,8 +332,8 @@ int getAuthType(int hSocket, _MODULE_DATA* _psSessionData, sLogin** login, int n
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
-  sprintf(bufSend, "GET /%s HTTP/1.0\r\nHost: %s:%d\r\nUser-Agent: %s\r\n\r\n", 
-          _psSessionData->szDir, (*login)->psServer->pHostIP, nPort, _psSessionData->szUserAgent);
+  sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\n\r\n", 
+          _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent);
 
   writeError(ERR_DEBUG_MODULE, "[%s] Sending initial non-authentication request: %s", MODULE_NAME, bufSend);
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
@@ -395,7 +389,6 @@ int sendAuthBasic(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   unsigned char* bufSend;
   unsigned char* bufReceive;
   unsigned char* szEncodedAuth;
-  int nReceiveBufferSize = 0;
   int nSendBufferSize = 0;
   int nRet = SUCCESS;
 
@@ -409,7 +402,7 @@ int sendAuthBasic(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
-  sprintf(bufSend, "GET /%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: Basic %s\r\n\r\n", 
+  sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: Basic %s\r\n\r\n", 
           _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, szEncodedAuth);
   
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
@@ -451,7 +444,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
-  sprintf(bufSend, "GET /%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: %s\r\nConnection: Keep-Alive\r\n\r\n", 
+  sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: %s\r\nConnection: close\r\n\r\n", 
           _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, szEncodedAuth);
   
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
@@ -516,7 +509,7 @@ int sendAuthNTLM(int hSocket, _MODULE_DATA* _psSessionData, char* szLogin, char*
   bufSend = malloc(nSendBufferSize + 1);
   memset(bufSend, 0, nSendBufferSize + 1);
 
-  sprintf(bufSend, "GET /%s HTTP/1.0\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: %s\r\nConnection: Keep-Alive\r\n\r\n", 
+  sprintf(bufSend, "GET /%s HTTP/1.1\r\nHost: %s\r\nUser-Agent: %s\r\nAuthorization: %s\r\nConnection: close\r\n\r\n", 
           _psSessionData->szDir, _psSessionData->szHostHeader, _psSessionData->szUserAgent, szEncodedAuth);
   
   if (medusaSend(hSocket, bufSend, nSendBufferSize, 0) < 0)
