@@ -23,6 +23,14 @@
 **
 **    MySQL 4.1 authentication support added by: pmonkey <pmonkey@foofus.net>
 **
+**    MySQL <4.1 Pass Hash Support: jmk <jmk@foofus.net>
+**
+**    The older MySQL pre-4.1 authentication scheme is vulnerable to a pass-the-
+**    hash authentication attack. Utilizing the old-style hashes gathered from a
+**    MySQL database, a user can use Medusa to verify their validity on other 
+**    servers. A modified MySQL client can also be use to connect to located 
+**    services directly utilizing a valid hash.
+**
 */
 
 #include <sys/types.h>
@@ -36,8 +44,8 @@
 #define MODULE_NAME    "mysql.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for MySQL sessions"
-#define MODULE_VERSION    "1.1.2"
-#define MODULE_VERSION_SVN "$Id: mysql.c 606 2006-10-19 20:22:41Z jmk $"
+#define MODULE_VERSION    "1.2"
+#define MODULE_VERSION_SVN "$Id: mysql.c 811 2007-08-28 21:19:29Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define BUF_SIZE 300
 
@@ -50,9 +58,12 @@
 #define PORT_MYSQL 3306
 #define PROTO_NEW 0
 #define PROTO_OLD 1
+#define PASSWORD 1
+#define HASH 2
 
 typedef struct __MYSQL_DATA {
   int protoFlag;
+  int hashFlag;
 } _MYSQL_DATA;
 
 
@@ -98,12 +109,27 @@ void summaryUsage(char **ppszSummary)
 void showUsage()
 {
   writeVerbose(VB_NONE, "%s (%s) %s :: %s\n", MODULE_NAME, MODULE_VERSION, MODULE_AUTHOR, MODULE_SUMMARY_USAGE);
+  writeVerbose(VB_NONE, "Available module options:");
+  writeVerbose(VB_NONE, "  PASS:?  (PASSWORD*, HASH)");
+  writeVerbose(VB_NONE, "    PASSWORD: Use normal password.");
+  writeVerbose(VB_NONE, "    HASH:     Use a hash rather than a password. (non-SHA1 hashes only)");
+  writeVerbose(VB_NONE, "\n(*) Default value");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "Usage examples:");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "1: Normal boring check...");
+  writeVerbose(VB_NONE, "    medusa -M mysql -h somehost -u someuser -p somepassword");
+  writeVerbose(VB_NONE, "");
+  writeVerbose(VB_NONE, "2: Using an old-style MySQL hash...");
+  writeVerbose(VB_NONE, "    medusa -M mysql -h somehost -U users.txt -p 39b52a209cf03d62 -m PASS:HASH");
+  writeVerbose(VB_NONE, "");
 }
 
 // The "main" of the medusa module world - this is what gets called to actually do the work
 int go(sLogin* logins, int argc, char *argv[])
 {
   int i;
+  char *strtok_ptr = NULL, *pOpt = NULL, *pOptTmp = NULL;
   _MYSQL_DATA *psSessionData;
 
   psSessionData = malloc(sizeof(_MYSQL_DATA));
@@ -119,6 +145,31 @@ int go(sLogin* logins, int argc, char *argv[])
   {
     writeError(ERR_DEBUG_MODULE, "OMG teh %s module has been called!!", MODULE_NAME);
 
+    psSessionData->hashFlag = PASSWORD;
+
+    for (i=0; i<argc; i++) {
+      pOptTmp = malloc( strlen(argv[i]) + 1);
+      memset(pOptTmp, 0, strlen(argv[i]) + 1);
+      strncpy(pOptTmp, argv[i], strlen(argv[i]));
+      writeError(ERR_DEBUG_MODULE, "Processing complete option: %s", pOptTmp);
+      pOpt = strtok_r(pOptTmp, ":", &strtok_ptr);
+      writeError(ERR_DEBUG_MODULE, "Processing option: %s", pOpt);
+
+      if (strcmp(pOpt, "PASS") == 0) {
+        pOpt = strtok_r(NULL, "\0", &strtok_ptr);
+        writeError(ERR_DEBUG_MODULE, "Processing option parameter: %s", pOpt);
+
+        if (strcmp(pOpt, "PASSWORD") == 0)
+          psSessionData->hashFlag = PASSWORD;
+        else if (strcmp(pOpt, "HASH") == 0)
+          psSessionData->hashFlag = HASH;
+        else
+          writeError(ERR_WARNING, "Invalid value for method PASS.");
+      }
+
+      free(pOptTmp);
+    }
+
     initModule(logins, psSessionData);
   }  
 
@@ -323,7 +374,7 @@ double my_rnd(struct rand_struct *rand_st)
   return (((double) rand_st->seed1)/rand_st->max_value_dbl);
 }
 
-void scramble_323(char *to, const char *message, const char *password)
+void scramble_323(char *to, _MYSQL_DATA *_psSessionData, const char *message, const char *password)
 {
   struct rand_struct rand_st;
   ulong hash_pass[2], hash_message[2];
@@ -332,7 +383,18 @@ void scramble_323(char *to, const char *message, const char *password)
   {
     char extra, *to_start=to;
     const char *message_end= message + SCRAMBLE_LENGTH_323;
-    hash_password(hash_pass,password, strlen(password));
+    
+    /* Idea borrowed from "The Database Hacker's Handbook: Defending Database Servers" */
+    if (_psSessionData->hashFlag == HASH)
+    {
+      if (strlen(password) != 16)
+        writeError(ERR_ERROR, "[%s] Invalid Hash Type (Old Style Hash Required)", MODULE_NAME);
+
+      sscanf(password, "%08lx%08lx", &hash_pass[0], &hash_pass[1]);
+    }
+    else
+      hash_password(hash_pass, password, strlen(password));
+
     hash_password(hash_message, message, SCRAMBLE_LENGTH_323);
     randominit(&rand_st,hash_pass[0] ^ hash_message[0],
                hash_pass[1] ^ hash_message[1]);
@@ -345,13 +407,23 @@ void scramble_323(char *to, const char *message, const char *password)
   *to= 0;
 }
 
-void scramble(char *to, const char *message, const char *password)
+void scramble(char *to, _MYSQL_DATA *_psSessionData, const char *message, const char *password)
 {
   SHA1_CONTEXT sha1_context;
   uint8 hash_stage1[SHA1_HASH_SIZE];
   uint8 hash_stage2[SHA1_HASH_SIZE];
 
   sha1_reset(&sha1_context);
+  
+  /* Stock pass-the-hash Attacks do not appear feasible. However, if another conversation is
+     intercepted and the SHA1 hash (hash_stage2) is known, hash_stage1 could probably be
+     derived. A new reply could then be generated based on that information. Of course, if
+     we already have this sort of network access, what are we messing with this for?
+  */
+  if (_psSessionData->hashFlag == HASH)
+    if ( (strncmp(password, "*", 1) == 0) && (strlen(password) == 2 * SHA1_HASH_SIZE + 1) )  
+      writeError(ERR_ERROR, "[%s] MySQL 4.1 and above use a SHA1-based authentication scheme which does not appear to be susceptible to pass-the-hash style attacks.", MODULE_NAME);
+
   /* stage 1: hash password */
   sha1_input(&sha1_context, (uint8 *) password, strlen(password));
   sha1_result(&sha1_context, hash_stage1);
@@ -370,7 +442,7 @@ void scramble(char *to, const char *message, const char *password)
 
 /* End of MySQL copy and paste items */ 
 
-int MySQLPrepareAuthOld(char* szLogin, char* szPassword, char* szSessionSalt, unsigned char** szResponse, unsigned long* iResponseLength)
+int MySQLPrepareAuthOld(_MYSQL_DATA *_psSessionData, char* szLogin, char* szPassword, char* szSessionSalt, unsigned char** szResponse, unsigned long* iResponseLength)
 {
   char* szHash;
   unsigned long login_len = strlen(szLogin) > 32 ? 32 : strlen(szLogin);
@@ -402,7 +474,7 @@ int MySQLPrepareAuthOld(char* szLogin, char* szPassword, char* szSessionSalt, un
   response[9 + login_len] = '\0';       /* null terminate login */
   
   if ( strcmp(szPassword, "") )
-    scramble_323((char *) &response[9 + login_len + 1], szSessionSalt, szPassword);
+    scramble_323((char *) &response[9 + login_len + 1], _psSessionData, szSessionSalt, szPassword);
 
   *szResponse = response;
   return SUCCESS;
@@ -410,7 +482,7 @@ int MySQLPrepareAuthOld(char* szLogin, char* szPassword, char* szSessionSalt, un
 
 /* This is a weird packet format used on a rare case when the newer Mysql
    still has a password table entry in the old shorter format */
-int MySQLPrepareAuthNewOld(char* szLogin, char* szPassword, char* szSessionSalt, unsigned char** szResponse, unsigned long* iResponseLength)
+int MySQLPrepareAuthNewOld(_MYSQL_DATA *_psSessionData, char* szLogin, char* szPassword, char* szSessionSalt, unsigned char** szResponse, unsigned long* iResponseLength)
 {
   char* szHash;
   unsigned long login_len = strlen(szLogin) > 32 ? 32 : strlen(szLogin);
@@ -431,13 +503,13 @@ int MySQLPrepareAuthNewOld(char* szLogin, char* szPassword, char* szSessionSalt,
 
   *((unsigned long *) response) = response_len - 4;  /* packet length */
   response[3] = 0x03;           /* packet number */
-  scramble_323((char *) &response[4], szSessionSalt, szPassword);
+  scramble_323((char *) &response[4], _psSessionData, szSessionSalt, szPassword);
 
   *szResponse = response;
   return SUCCESS;
 }
 
-int MySQLPrepareAuth(char* szLogin, char* szPassword, char* szSessionSalt, unsigned char** szResponse, unsigned long* iResponseLength)
+int MySQLPrepareAuth(_MYSQL_DATA *_psSessionData, char* szLogin, char* szPassword, char* szSessionSalt, unsigned char** szResponse, unsigned long* iResponseLength)
 {
   char* szHash;
   unsigned long login_len = strlen(szLogin) > 32 ? 32 : strlen(szLogin);
@@ -487,7 +559,7 @@ int MySQLPrepareAuth(char* szLogin, char* szPassword, char* szSessionSalt, unsig
     response[36+strlen(szLogin)+1] = 0x14;
 
     /* Now add the scramble SHA hash */
-    scramble((char *) &response[36+strlen(szLogin)+2], szSessionSalt, szPassword); 
+    scramble((char *) &response[36+strlen(szLogin)+2], _psSessionData, szSessionSalt, szPassword); 
   }
   
   *szResponse = response;
@@ -511,9 +583,18 @@ int MySQLSessionInit(int hSocket, unsigned char** szSessionSalt)
   /* check protocol version */
   if (bufReceive[4] == 0xff)
   {
-    writeError(ERR_ERROR, "%s: Failed to retrieve server version: %s", MODULE_NAME, bufReceive + 7);
-    FREE(bufReceive);
-    return FAILURE;
+    if (strstr(bufReceive + 7, "is not allowed to connect to this MySQL server"))
+    {
+      writeError(ERR_WARNING, "%s: Server responded that host is not allowed to connect to MySQL service.", MODULE_NAME);
+      FREE(bufReceive);
+      return FAILURE;
+    }
+    else
+    {
+      writeError(ERR_ERROR, "%s: Failed to retrieve server version: %s", MODULE_NAME, bufReceive + 7);
+      FREE(bufReceive);
+      return FAILURE;
+    }
   }
 
   if (bufReceive[4] < 10)
@@ -599,8 +680,10 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
   iRet = MySQLSessionInit(hSocket, &szSessionSalt);
   if (iRet == FAILURE)
   {
-    writeError(ERR_ERROR, "%s: Failed to initialize MySQL connection.", MODULE_NAME);
-    return FAILURE;
+    writeError(ERR_ERROR, "%s: Failed to initialize MySQL connection (%s).", MODULE_NAME, (*psLogin)->psServer->pHostIP);
+    (*psLogin)->iResult = LOGIN_RESULT_ERROR;
+    (*psLogin)->iStatus = LOGIN_FAILED;
+    return MSTATE_EXITING;
   }
 
   /* prepare client authentication packet */
@@ -610,7 +693,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
       writeError(ERR_DEBUG, "%s: using older auth per previous attempt response.", MODULE_NAME);
     }
     
-    iRet = MySQLPrepareAuthOld(szLogin, szPassword, szSessionSalt, &szResponse, &iResponseLength);
+    iRet = MySQLPrepareAuthOld(_psSessionData, szLogin, szPassword, szSessionSalt, &szResponse, &iResponseLength);
     if (iRet == FAILURE)
     {
       writeError(ERR_ERROR, "%s: Failed to create client authentication packet.", MODULE_NAME);
@@ -619,7 +702,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
   } 
   else 
   {
-    iRet = MySQLPrepareAuth(szLogin, szPassword, szSessionSalt, &szResponse, &iResponseLength);
+    iRet = MySQLPrepareAuth(_psSessionData, szLogin, szPassword, szSessionSalt, &szResponse, &iResponseLength);
     if (iRet == FAILURE)
     {
       writeError(ERR_ERROR, "%s: Failed to create client authentication packet.", MODULE_NAME);
@@ -676,7 +759,7 @@ int tryLogin(int hSocket, sLogin** psLogin, _MYSQL_DATA* _psSessionData, char* s
 
     /* Well, you gotta send just the old style scramble using the existing connection */
 
-    iRet = MySQLPrepareAuthNewOld(szLogin, szPassword, szSessionSalt, &szResponse,&iResponseLength);
+    iRet = MySQLPrepareAuthNewOld(_psSessionData, szLogin, szPassword, szSessionSalt, &szResponse,&iResponseLength);
     if (iRet == FAILURE)
     {
       writeError(ERR_ERROR, "%s: Failed to create client authentication packet.", MODULE_NAME);
