@@ -85,8 +85,8 @@
 #define MODULE_NAME    "smbnt.mod"
 #define MODULE_AUTHOR  "JoMo-Kun <jmk@foofus.net>"
 #define MODULE_SUMMARY_USAGE  "Brute force module for SMB/NTLMv1 sessions"
-#define MODULE_VERSION    "1.3.0"
-#define MODULE_VERSION_SVN "$Id: smbnt.c 606 2006-10-19 20:22:41Z jmk $"
+#define MODULE_VERSION    "1.3.1"
+#define MODULE_VERSION_SVN "$Id: smbnt.c 650 2007-01-23 18:50:35Z jmk $"
 #define MODULE_SUMMARY_FORMAT  "%s : version %s"
 #define MODULE_SUMMARY_FORMAT_WARN  "%s : version %s (%s)"
 #define OPENSSL_WARNING "No usable OPENSSL. Module disabled."
@@ -207,8 +207,8 @@ void showUsage()
 int go(sLogin* logins, int argc, char *argv[])
 {
   int i;
-  char *strtok_ptr, *pOpt, *pOptTmp;
-  _SMBNT_DATA *psSessionData;
+  char *strtok_ptr = NULL, *pOpt = NULL, *pOptTmp = NULL;
+  _SMBNT_DATA *psSessionData = NULL;
   psSessionData = malloc(sizeof(_SMBNT_DATA));  
   memset(psSessionData, 0, sizeof(_SMBNT_DATA));
 
@@ -295,11 +295,11 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
 {
   int hSocket = -1;
   enum MODULE_STATE nState = MSTATE_NEW;
-  char* bufReceive;
+  char *bufReceive = NULL;
   int nReceiveBufferSize = 0, nFirstPass = 0, nFoundPrompt = 0;
   int i = 0;
-  char *pPass;
-  sUser* user = psLogin->psUser;
+  char *pPass = NULL;
+  sUser *user = psLogin->psUser;
   sConnectParams params;
 
   memset(&params, 0, sizeof(sConnectParams));
@@ -350,7 +350,7 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
         
         if (hSocket < 0) 
         {
-          writeError(ERR_NOTICE, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
+          writeError(ERR_ERROR, "%s: failed to connect, port %d was not open on %s", MODULE_NAME, params.nPort, psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
           psLogin->iStatus = LOGIN_FAILED;
           free(_psSessionData);
@@ -360,7 +360,7 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
         writeError(ERR_DEBUG_MODULE, "Connected");
  
         if (NBSSessionRequest(hSocket, _psSessionData) < 0) {
-          writeError(ERR_DEBUG_MODULE, "Session Setup Failed with host: %s. Is the server service running?", psLogin->psServer->pHostIP);
+          writeError(ERR_ERROR, "Session Setup Failed with host: %s. Is the server service running?", psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
           psLogin->iStatus = LOGIN_FAILED;
           free(_psSessionData);
@@ -369,7 +369,7 @@ int initModule(sLogin* psLogin, _SMBNT_DATA *_psSessionData)
         
         if (SMBNegProt(hSocket, _psSessionData) < 0)
         {
-          writeError(ERR_DEBUG_MODULE, "SMB Protocol Negotiation Failed with host: %s", psLogin->psServer->pHostIP);
+          writeError(ERR_ERROR, "SMB Protocol Negotiation Failed with host: %s", psLogin->psServer->pHostIP);
           psLogin->iResult = LOGIN_RESULT_UNKNOWN;
           psLogin->iStatus = LOGIN_FAILED;
           free(_psSessionData);
@@ -465,7 +465,7 @@ int HashNTLM(_SMBNT_DATA *_psSessionData, unsigned char **ntlmhash, unsigned cha
   unsigned char ntlm_response[24];
   int i = 0, j = 0;
   int mdlen;
-  unsigned char *p;
+  unsigned char *p = NULL;
   char HexChar;
   int HexValue;
 
@@ -573,7 +573,7 @@ int NBSSessionRequest(int hSocket, _SMBNT_DATA* _psSessionData)
   char nb_local[32];            /* netbios localredirector */
   unsigned char rqbuf[7] = { 0x81, 0x00, 0x00, 0x48, 0x20, 0x00, 0x20 };
   char *buf = NULL;
-  char* bufReceive = NULL;
+  char *bufReceive = NULL;
   int nReceiveBufferSize = 0;
   
   /* if we are running in native mode (aka port 445) don't do netbios */
@@ -770,9 +770,9 @@ unsigned long SMBSessionSetup(int hSocket, _SMBNT_DATA *_psSessionData, char* sz
   };
 
   unsigned char buf[512];
-  unsigned char *NTLMhash;
+  unsigned char *NTLMhash = NULL;
   unsigned char LMhash[24];
-  char* bufReceive;
+  char *bufReceive = NULL;
   int nReceiveBufferSize = 0;
   unsigned char sess_key[2];
   int i, userlen, passlen, ret;
@@ -913,8 +913,8 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
 {
   int SMBerr, SMBaction;
   unsigned long SMBSessionRet;
-  char *pErrorMsg;
-  char ErrorCode[8];
+  char *pErrorMsg = NULL;
+  char ErrorCode[10];
   int i, iRet;
 
   /* Nessus Plugins: smb_header.inc */
@@ -957,6 +957,8 @@ int tryLogin(int hSocket, sLogin** psLogin, _SMBNT_DATA* _psSessionData, char* s
     "AS400_STATUS_LOGON_FAILURE"
   };
 
+  memset(&ErrorCode, 0, 10);
+
   SMBSessionRet = SMBSessionSetup(hSocket, _psSessionData, szLogin, szPassword);
   SMBerr = (unsigned long) SMBSessionRet & 0x00FFFFFF;
   SMBaction = ((unsigned long) SMBSessionRet & 0xFF000000) >> 24;
